@@ -1,4 +1,4 @@
-// Copyright (C) 2007-2011 Anders Logg
+// Copyright (C) 2007-2015 Anders Logg
 //
 // This file is part of DOLFIN.
 //
@@ -18,9 +18,7 @@
 // Modified by Garth N. Wells 2007-2008
 // Modified by Ola Skavhaug 2008
 // Modified by Joachim B Haga 2012
-//
-// First added:  2007-01-17
-// Last changed: 2013-09-19
+// Modified by Martin Alnes 2015
 
 #ifndef __ASSEMBLER_H
 #define __ASSEMBLER_H
@@ -87,6 +85,7 @@ namespace dolfin
     void assemble_interior_facets(GenericTensor& A, const Form& a,
                                   UFC& ufc,
                                   std::shared_ptr<const MeshFunction<std::size_t> > domains,
+                                  std::shared_ptr<const MeshFunction<std::size_t> > cell_domains,
                                   std::vector<double>* values);
 
     /// Assemble tensor from given form over vertices. This function is
