@@ -17,6 +17,7 @@
 //
 // Modified by Joachim B. Haga, 2012
 // Modified by Jan Blechta, 2013
+// Modified by Martin Alnes, 2015
 
 #ifndef __GENERIC_DOF_MAP_H
 #define __GENERIC_DOF_MAP_H
@@ -29,6 +30,8 @@
 #include <unordered_map>
 #include <unordered_set>
 
+#include <dolfin/common/Array.h>
+#include <dolfin/common/ArrayView.h>
 #include <dolfin/common/types.h>
 #include <dolfin/common/Variable.h>
 #include <dolfin/log/log.h>
@@ -67,29 +70,33 @@ namespace dolfin
     /// process
     virtual std::size_t local_dimension(std::string type) const = 0;
 
+    /// Return the dimension of the local finite element function
+    /// space on a cell (deprecated API)
+    std::size_t cell_dimension(std::size_t index) const
+    {
+      // TODO: Add deprecation warning
+      return num_element_dofs(index);
+    }
+
+    /// Return the maximum dimension of the local finite element
+    /// function space (deprecated API)
+    std::size_t max_cell_dimension() const
+    {
+      // TODO: Add deprecation warning
+      return max_element_dofs();
+    }
+
     /// Return the dimension of the local finite element function
     /// space on a cell
-    virtual std::size_t cell_dimension(std::size_t index) const = 0;
+    virtual std::size_t num_element_dofs(std::size_t index) const = 0;
 
     /// Return the maximum dimension of the local finite element
     /// function space
-    virtual std::size_t max_cell_dimension() const = 0;
+    virtual std::size_t max_element_dofs() const = 0;
 
     /// Return the number of dofs for a given entity dimension
     virtual std::size_t num_entity_dofs(std::size_t dim) const = 0;
 
-    // DEPRECATED
-    /// Return the geometric dimension of the coordinates this dof map
-    /// provides
-    virtual std::size_t geometric_dimension() const
-    {
-      // Throw error if not provided by sub-class
-      dolfin_error("GenericDofMap.cpp",
-                   "get dofmap geometric dimension",
-                   "dofmap generic dimension is deprecated and not supported by this implementation of GenericDofMap interface.");
-      return 0;
-    }
-
     /// Return number of facet dofs
     virtual std::size_t num_facet_dofs() const = 0;
 
@@ -102,8 +109,8 @@ namespace dolfin
     virtual const std::vector<int>& off_process_owner() const = 0;
 
     /// Local-to-global mapping of dofs on a cell
-    virtual const std::vector<dolfin::la_index>&
-      cell_dofs(std::size_t cell_index) const = 0;
+    virtual ArrayView<const dolfin::la_index>
+    cell_dofs(std::size_t cell_index) const = 0;
 
     /// Tabulate local-local facet dofs
     virtual void tabulate_facet_dofs(std::vector<std::size_t>& dofs,
@@ -112,7 +119,7 @@ namespace dolfin
     /// Tabulate the local-to-local mapping of dofs on entity
     /// (dim, local_entity)
     virtual void tabulate_entity_dofs(std::vector<std::size_t>& dofs,
-				      std::size_t dim,
+                                      std::size_t dim,
                                       std::size_t local_entity) const = 0;
 
     /// Tabulate the coordinates of all dofs on a cell (UFC cell version)
@@ -146,6 +153,11 @@ namespace dolfin
         collapse(std::unordered_map<std::size_t, std::size_t>& collapsed_map,
                  const Mesh& mesh) const = 0;
 
+    /// Return list of dof indices on this process that belong to mesh
+    /// entities of dimension dim
+    virtual std::vector<dolfin::la_index> dofs(const Mesh& mesh,
+                                               std::size_t dim) const = 0;
+
     /// Return list of global dof indices on this process
     virtual std::vector<dolfin::la_index> dofs() const = 0;
 
