@@ -16,9 +16,11 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // First added:  2011-04-13
-// Last changed: 2013-08-28
+// Last changed: 2014-03-25
 
-#include <boost/scoped_array.hpp>
+#include <limits>
+#include <memory>
+#include <vector>
 
 #include <dolfin/common/NoDeleter.h>
 #include <dolfin/function/FunctionSpace.h>
@@ -36,16 +38,16 @@ using namespace dolfin;
 PointSource::PointSource(const FunctionSpace& V,
                          const Point& p,
                          double magnitude)
-  : _V(reference_to_no_delete_pointer(V)), _p(p), _magnitude(magnitude)
+  : _function_space(reference_to_no_delete_pointer(V)), _p(p), _magnitude(magnitude)
 {
   // Check that function space is scalar
   check_is_scalar(V);
 }
 //-----------------------------------------------------------------------------
-PointSource::PointSource(boost::shared_ptr<const FunctionSpace> V,
+PointSource::PointSource(std::shared_ptr<const FunctionSpace> V,
                          const Point& p,
                          double magnitude)
-  : _V(V), _p(p), _magnitude(magnitude)
+  : _function_space(V), _p(p), _magnitude(magnitude)
 {
   // Check that function space is scalar
   check_is_scalar(*V);
@@ -58,25 +60,25 @@ PointSource::~PointSource()
 //-----------------------------------------------------------------------------
 void PointSource::apply(GenericVector& b)
 {
-  dolfin_assert(_V);
+  dolfin_assert(_function_space);
 
   log(PROGRESS, "Applying point source to right-hand side vector.");
 
   // Find the cell containing the point (may be more than one cell but
   // we only care about the first). Well-defined if the basis
   // functions are continuous but may give unexpected results for DG.
-  dolfin_assert(_V->mesh());
-  const Mesh& mesh = *_V->mesh();
-  boost::shared_ptr<BoundingBoxTree> tree = mesh.bounding_box_tree();
+  dolfin_assert(_function_space->mesh());
+  const Mesh& mesh = *_function_space->mesh();
+  std::shared_ptr<BoundingBoxTree> tree = mesh.bounding_box_tree();
   const unsigned int cell_index = tree->compute_first_entity_collision(_p);
 
   // Check that we found the point on at least one processor
   int num_found = 0;
   if (cell_index == std::numeric_limits<unsigned int>::max())
-    num_found = MPI::sum(0);
+    num_found = MPI::sum(mesh.mpi_comm(), 0);
   else
-    num_found = MPI::sum(1);
-  if (MPI::process_number() == 0 && num_found == 0)
+    num_found = MPI::sum(mesh.mpi_comm(), 1);
+  if (MPI::rank(mesh.mpi_comm()) == 0 && num_found == 0)
   {
     dolfin_error("PointSource.cpp",
                  "apply point source to vector",
@@ -103,28 +105,31 @@ void PointSource::apply(GenericVector& b)
       vertex_coordinates[i*gdim + j] = mesh.geometry().x(vertices[i])[j];
 
   // Evaluate all basis functions at the point()
-  dolfin_assert(_V->element());
-  dolfin_assert(_V->element()->value_rank() == 0);
-  std::vector<double> values(_V->element()->space_dimension());
-  const int cell_orientation = 0;
-  _V->element()->evaluate_basis_all(values.data(),
-                                   _p.coordinates(),
-                                    vertex_coordinates.data(),
-                                    cell_orientation);
+  dolfin_assert(_function_space->element());
+  dolfin_assert(_function_space->element()->value_rank() == 0);
+  std::vector<double> values(_function_space->element()->space_dimension());
+
+  ufc::cell ufc_cell;
+  cell.get_cell_data(ufc_cell);
+  _function_space->element()->evaluate_basis_all(values.data(),
+                                                 _p.coordinates(),
+                                                 vertex_coordinates.data(),
+                                                 ufc_cell.orientation);
 
   // Scale by magnitude
-  for (std::size_t i = 0; i < _V->element()->space_dimension(); i++)
+  for (std::size_t i = 0; i < _function_space->element()->space_dimension(); i++)
     values[i] *= _magnitude;
 
   // Compute local-to-global mapping
-  dolfin_assert(_V->dofmap());
-  const std::vector<dolfin::la_index>& dofs
-    = _V->dofmap()->cell_dofs(cell.index());
+  dolfin_assert(_function_space->dofmap());
+  const ArrayView<const dolfin::la_index> dofs
+    = _function_space->dofmap()->cell_dofs(cell.index());
 
   // Add values to vector
-  dolfin_assert(_V->element()->space_dimension()
-                == _V->dofmap()->cell_dimension(cell.index()));
-  b.add(values.data(), _V->element()->space_dimension(), dofs.data());
+  dolfin_assert(_function_space->element()->space_dimension()
+                == _function_space->dofmap()->num_element_dofs(cell.index()));
+  b.add_local(values.data(), _function_space->element()->space_dimension(),
+              dofs.data());
   b.apply("add");
 }
 //-----------------------------------------------------------------------------
