@@ -1,4 +1,4 @@
-// Copyright (C) 2007-2013 Anders Logg
+// Copyright (C) 2007-2014 Anders Logg
 //
 // This file is part of DOLFIN.
 //
@@ -18,13 +18,13 @@
 // Modified by Garth N. Wells 2009
 //
 // First added:  2007-01-17
-// Last changed: 2013-02-26
+// Last changed: 2014-04-24
 
 #ifndef __UFC_DATA_H
 #define __UFC_DATA_H
 
 #include <vector>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include <ufc.h>
 
 namespace dolfin
@@ -62,9 +62,28 @@ namespace dolfin
     /// Update current cell
     void update(const Cell& cell,
                 const std::vector<double>& vertex_coordinates0,
-                const ufc::cell& ufc_cell);
+                const ufc::cell& ufc_cell,
+                const std::vector<bool> & enabled_coefficients);
 
     /// Update current pair of cells for macro element
+    void update(const Cell& cell0,
+                const std::vector<double>& vertex_coordinates0,
+                const ufc::cell& ufc_cell0,
+                const Cell& cell1,
+                const std::vector<double>& vertex_coordinates1,
+                const ufc::cell& ufc_cell1,
+                const std::vector<bool> & enabled_coefficients);
+
+    /// Update current cell (TODO: Remove this when
+    /// PointIntegralSolver supports the version with
+    /// enabled_coefficients)
+    void update(const Cell& cell,
+                const std::vector<double>& vertex_coordinates0,
+                const ufc::cell& ufc_cell);
+
+    /// Update current pair of cells for macro element (TODO: Remove
+    /// this when PointIntegralSolver supports the version with
+    /// enabled_coefficients)
     void update(const Cell& cell0,
                 const std::vector<double>& vertex_coordinates0,
                 const ufc::cell& ufc_cell0,
@@ -74,64 +93,77 @@ namespace dolfin
 
     /// Pointer to coefficient data. Used to support UFC interface.
     const double* const * w() const
-    { return &w_pointer[0]; }
+    { return w_pointer.data(); }
 
     /// Pointer to coefficient data. Used to support UFC
     /// interface. None const version
     double* * w()
-    { return &w_pointer[0]; }
+    { return w_pointer.data(); }
 
     /// Pointer to macro element coefficient data. Used to support UFC
     /// interface.
     const double* const * macro_w() const
-    { return &macro_w_pointer[0]; }
+    { return macro_w_pointer.data(); }
 
   private:
 
     // Finite elements for coefficients
     std::vector<FiniteElement> coefficient_elements;
 
-    // Cell integrals (access through get_cell_integral to get proper
-    // fallback to default)
-    std::vector<boost::shared_ptr<ufc::cell_integral> > cell_integrals;
+    // Cell integrals (access through
+    // get_cell_integral to get proper fallback to default)
+    std::vector<std::shared_ptr<ufc::cell_integral> >
+      cell_integrals;
 
     // Exterior facet integrals (access through
     // get_exterior_facet_integral to get proper fallback to default)
-    std::vector<boost::shared_ptr<ufc::exterior_facet_integral> >
+    std::vector<std::shared_ptr<ufc::exterior_facet_integral> >
       exterior_facet_integrals;
 
     // Interior facet integrals (access through
     // get_interior_facet_integral to get proper fallback to default)
-    std::vector<boost::shared_ptr<ufc::interior_facet_integral> >
+    std::vector<std::shared_ptr<ufc::interior_facet_integral> >
       interior_facet_integrals;
 
-    // Point integrals (access through get_point_integral to get
+    // Point integrals (access through get_vertex_integral to get
     // proper fallback to default)
-    std::vector<boost::shared_ptr<ufc::point_integral> > point_integrals;
+    std::vector<std::shared_ptr<ufc::vertex_integral> >
+      vertex_integrals;
+
+    // Custom integrals (access through get_custom_integral to get
+    // proper fallback to default)
+    std::vector<std::shared_ptr<ufc::custom_integral> > custom_integrals;
 
   public:
 
     // Default cell integral
-    boost::shared_ptr<ufc::cell_integral> default_cell_integral;
+    std::shared_ptr<ufc::cell_integral>
+      default_cell_integral;
 
     // Default exterior facet integral
-    boost::shared_ptr<ufc::exterior_facet_integral>
+    std::shared_ptr<ufc::exterior_facet_integral>
       default_exterior_facet_integral;
 
     // Default interior facet integral
-    boost::shared_ptr<ufc::interior_facet_integral>
+    std::shared_ptr<ufc::interior_facet_integral>
       default_interior_facet_integral;
 
     // Default point integral
-    boost::shared_ptr<ufc::point_integral> default_point_integral;
+    std::shared_ptr<ufc::vertex_integral>
+      default_vertex_integral;
+
+    // Default custom integral
+    std::shared_ptr<ufc::custom_integral> default_custom_integral;
 
     /// Get cell integral over a given domain, falling back to the
     /// default if necessary
-    ufc::cell_integral * get_cell_integral(std::size_t domain)
+    ufc::cell_integral*
+      get_cell_integral(std::size_t domain)
     {
-      if (domain < form.num_cell_domains())
+      if (domain < form.max_cell_subdomain_id())
       {
-        ufc::cell_integral * integral = cell_integrals[domain].get();
+        ufc::cell_integral * integral
+          = cell_integrals[domain].get();
         if (integral)
           return integral;
       }
@@ -143,7 +175,7 @@ namespace dolfin
     ufc::exterior_facet_integral*
       get_exterior_facet_integral(std::size_t domain)
     {
-      if (domain < form.num_exterior_facet_domains())
+      if (domain < form.max_exterior_facet_subdomain_id())
       {
         ufc::exterior_facet_integral* integral
           = exterior_facet_integrals[domain].get();
@@ -158,7 +190,7 @@ namespace dolfin
     ufc::interior_facet_integral*
       get_interior_facet_integral(std::size_t domain)
     {
-      if (domain < form.num_interior_facet_domains())
+      if (domain < form.max_interior_facet_subdomain_id())
       {
         ufc::interior_facet_integral* integral
           = interior_facet_integrals[domain].get();
@@ -170,15 +202,30 @@ namespace dolfin
 
     /// Get point integral over a given domain, falling back to the
     /// default if necessary
-    ufc::point_integral * get_point_integral(std::size_t domain)
+    ufc::vertex_integral*
+      get_vertex_integral(std::size_t domain)
     {
-      if (domain < form.num_point_domains())
+      if (domain < form.max_vertex_subdomain_id())
       {
-        ufc::point_integral * integral = point_integrals[domain].get();
+        ufc::vertex_integral * integral
+          = vertex_integrals[domain].get();
         if (integral)
           return integral;
       }
-      return default_point_integral.get();
+      return default_vertex_integral.get();
+    }
+
+    /// Get custom integral over a given domain, falling back to the
+    /// default if necessary
+    ufc::custom_integral * get_custom_integral(std::size_t domain)
+    {
+      if (domain < form.max_custom_subdomain_id())
+      {
+        ufc::custom_integral * integral = custom_integrals[domain].get();
+        if (integral)
+          return integral;
+      }
+      return default_custom_integral.get();
     }
 
     // Form
@@ -208,7 +255,7 @@ namespace dolfin
     std::vector<double*> macro_w_pointer;
 
     // Coefficient functions
-    const std::vector<boost::shared_ptr<const GenericFunction> > coefficients;
+    const std::vector<std::shared_ptr<const GenericFunction> > coefficients;
 
   public:
 
