@@ -45,8 +45,9 @@ FunctionAssigner::FunctionAssigner(std::shared_ptr<const FunctionSpace> receivin
   _check_and_build_indices(mesh, _receiving_spaces, _assigning_spaces);
 }
 //-----------------------------------------------------------------------------
-FunctionAssigner::FunctionAssigner(std::vector<std::shared_ptr<const FunctionSpace> > receiving_spaces,
-	           std::shared_ptr<const FunctionSpace> assigning_space)
+FunctionAssigner::FunctionAssigner(
+  std::vector<std::shared_ptr<const FunctionSpace>> receiving_spaces,
+  std::shared_ptr<const FunctionSpace> assigning_space)
   : _receiving_spaces(receiving_spaces), _assigning_spaces(1, assigning_space),
     _receiving_indices(receiving_spaces.size()),
     _assigning_indices(receiving_spaces.size()),
@@ -68,7 +69,7 @@ FunctionAssigner::FunctionAssigner(std::vector<std::shared_ptr<const FunctionSpa
   }
 
   // Collect assigning sub spaces
-  std::vector<std::shared_ptr<const FunctionSpace> > assigning_sub_spaces;
+  std::vector<std::shared_ptr<const FunctionSpace>> assigning_sub_spaces;
   for (std::size_t sub_space_ind = 0; sub_space_ind < N; sub_space_ind++)
     assigning_sub_spaces.push_back((*_assigning_spaces[0])[sub_space_ind]);
 
@@ -76,8 +77,9 @@ FunctionAssigner::FunctionAssigner(std::vector<std::shared_ptr<const FunctionSpa
   _check_and_build_indices(mesh, _receiving_spaces, assigning_sub_spaces);
 }
 //-----------------------------------------------------------------------------
-FunctionAssigner::FunctionAssigner(std::shared_ptr<const FunctionSpace> receiving_space,
-				   std::vector<std::shared_ptr<const FunctionSpace> > assigning_spaces)
+FunctionAssigner::FunctionAssigner(
+  std::shared_ptr<const FunctionSpace> receiving_space,
+  std::vector<std::shared_ptr<const FunctionSpace>> assigning_spaces)
   :_receiving_spaces(1, receiving_space), _assigning_spaces(assigning_spaces),
    _receiving_indices(assigning_spaces.size()),
    _assigning_indices(assigning_spaces.size()),
@@ -99,7 +101,7 @@ FunctionAssigner::FunctionAssigner(std::shared_ptr<const FunctionSpace> receivin
   }
 
   // Collect receiving sub spaces
-  std::vector<std::shared_ptr<const FunctionSpace> > receiving_sub_spaces;
+  std::vector<std::shared_ptr<const FunctionSpace>> receiving_sub_spaces;
   for (std::size_t sub_space_ind = 0; sub_space_ind < N; sub_space_ind++)
     receiving_sub_spaces.push_back((*_receiving_spaces[0])[sub_space_ind]);
 
@@ -117,16 +119,17 @@ FunctionAssigner::assign(std::shared_ptr<Function> receiving_func,
                          std::shared_ptr<const Function> assigning_func) const
 {
   // Wrap functions
-  std::vector<std::shared_ptr<Function> > receiving_funcs(1, receiving_func);
-  std::vector<std::shared_ptr<const Function> >
+  std::vector<std::shared_ptr<Function>> receiving_funcs(1, receiving_func);
+  std::vector<std::shared_ptr<const Function>>
     assigning_funcs(1, assigning_func);
 
   // Do the assignment
   _assign(receiving_funcs, assigning_funcs);
 }
 //-----------------------------------------------------------------------------
-void FunctionAssigner::assign(std::shared_ptr<Function> receiving_func,
-      std::vector<std::shared_ptr<const Function> > assigning_funcs) const
+void FunctionAssigner::assign(
+  std::shared_ptr<Function> receiving_func,
+  std::vector<std::shared_ptr<const Function>> assigning_funcs) const
 {
   // Num assigning functions
   const std::size_t N = assigning_funcs.size();
@@ -139,7 +142,7 @@ void FunctionAssigner::assign(std::shared_ptr<Function> receiving_func,
   }
 
   // Collect receiving sub functions
-  std::vector<std::shared_ptr<Function> > receiving_funcs(0);
+  std::vector<std::shared_ptr<Function>> receiving_funcs(0);
   for (std::size_t i = 0; i < N; i++)
   {
     std::shared_ptr<Function>
@@ -151,8 +154,9 @@ void FunctionAssigner::assign(std::shared_ptr<Function> receiving_func,
   _assign(receiving_funcs, assigning_funcs);
 }
 //-----------------------------------------------------------------------------
-void FunctionAssigner::assign(std::vector<std::shared_ptr<Function> > receiving_funcs,
-			      std::shared_ptr<const Function> assigning_func) const
+void
+FunctionAssigner::assign(std::vector<std::shared_ptr<Function>> receiving_funcs,
+                         std::shared_ptr<const Function> assigning_func) const
 {
   // Num receiving functions
   const std::size_t N = receiving_funcs.size();
@@ -165,7 +169,7 @@ void FunctionAssigner::assign(std::vector<std::shared_ptr<Function> > receiving_
   }
 
   // Collect receiving sub functions
-  std::vector<std::shared_ptr<const Function> > assigning_funcs(0);
+  std::vector<std::shared_ptr<const Function>> assigning_funcs(0);
   for (std::size_t i = 0; i < N; i++)
   {
     std::shared_ptr<const Function>
@@ -177,8 +181,9 @@ void FunctionAssigner::assign(std::vector<std::shared_ptr<Function> > receiving_
   _assign(receiving_funcs, assigning_funcs);
 }
 //-----------------------------------------------------------------------------
-void FunctionAssigner::_assign(std::vector<std::shared_ptr<Function> > receiving_funcs,
-			       std::vector<std::shared_ptr<const Function> > assigning_funcs) const
+void FunctionAssigner::_assign(
+  std::vector<std::shared_ptr<Function>> receiving_funcs,
+  std::vector<std::shared_ptr<const Function>> assigning_funcs) const
 {
   // Num spaces
   const std::size_t N = std::max(_assigning_spaces.size(),
@@ -373,9 +378,10 @@ const Mesh& FunctionAssigner::_get_mesh() const
   return mesh;
 }
 //-----------------------------------------------------------------------------
-void FunctionAssigner::_check_and_build_indices(const Mesh& mesh,
-	  const std::vector<std::shared_ptr<const FunctionSpace> >& receiving_spaces,
-	  const std::vector<std::shared_ptr<const FunctionSpace> >& assigning_spaces)
+void FunctionAssigner::_check_and_build_indices(
+  const Mesh& mesh,
+  const std::vector<std::shared_ptr<const FunctionSpace>>& receiving_spaces,
+  const std::vector<std::shared_ptr<const FunctionSpace>>& assigning_spaces)
 {
 
   // Num spaces
@@ -437,9 +443,9 @@ void FunctionAssigner::_check_and_build_indices(const Mesh& mesh,
     for (CellIterator cell(mesh); !cell.end(); ++cell)
     {
       // Get local cell dofs
-      const std::vector<dolfin::la_index>& assigning_cell_dofs
+      const ArrayView<const dolfin::la_index> assigning_cell_dofs
         = assigning_dofmap.cell_dofs(cell->index());
-      const std::vector<dolfin::la_index>& receiving_cell_dofs
+      const ArrayView<const dolfin::la_index> receiving_cell_dofs
         = receiving_dofmap.cell_dofs(cell->index());
 
       // Check that both spaces have the same number of dofs
