@@ -12,9 +12,11 @@
 #include <dolfin/function/Constant.h>
 #include <dolfin/function/SpecialFunctions.h>
 #include <dolfin/function/SpecialFacetFunction.h>
-#include <dolfin/function/CCFEMFunctionSpace.h>
+#include <dolfin/function/MultiMeshFunction.h>
+#include <dolfin/function/MultiMeshFunctionSpace.h>
+#include <dolfin/function/MultiMeshSubSpace.h>
 #include <dolfin/function/FunctionAssigner.h>
 #include <dolfin/function/assign.h>
-#include <dolfin/function/CCFEMFunction.h>
+#include <dolfin/function/LagrangeInterpolator.h>
 
 #endif
