@@ -11,5 +11,7 @@
 #include <dolfin/generation/UnitIntervalMesh.h>
 #include <dolfin/generation/UnitTriangleMesh.h>
 #include <dolfin/generation/UnitSquareMesh.h>
+#include <dolfin/generation/UnitQuadMesh.h>
+#include <dolfin/generation/UnitHexMesh.h>
 
 #endif
