@@ -1,4 +1,4 @@
-// Copyright (C) 2005 Johan Jansson
+// Copyright (C) 2014 Johan Jansson and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -18,14 +18,10 @@
 // Modified by Anders Logg 2005-2012
 // Modified by Garth N. Wells 2005-2010
 // Modified by Fredrik Valdmanis 2011
-//
-// First added:  2005-12-02
-// Last changed: 2013-11-25
 
 #ifdef HAS_PETSC
 
 #include <petsclog.h>
-#include <boost/assign/list_of.hpp>
 
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/NoDeleter.h>
@@ -43,98 +39,63 @@
 
 using namespace dolfin;
 
-// Utility functions
-namespace dolfin
-{
-  class PETScKSPDeleter
-  {
-  public:
-    void operator() (KSP* _ksp)
-    {
-      if (_ksp)
-        KSPDestroy(_ksp);
-      delete _ksp;
-    }
-  };
-}
-
-namespace dolfin
-{
-  class PETScMatNullSpaceDeleter
-  {
-  public:
-    void operator() (MatNullSpace* ns)
-    {
-      if (*ns)
-        MatNullSpaceDestroy(ns);
-      delete ns;
-    }
-  };
-}
-
 // Mapping from method string to PETSc
 const std::map<std::string, const KSPType> PETScKrylovSolver::_methods
-  = boost::assign::map_list_of("default",  "")
-                              ("cg",         KSPCG)
-                              ("gmres",      KSPGMRES)
-                              ("minres",     KSPMINRES)
-                              ("tfqmr",      KSPTFQMR)
-                              ("richardson", KSPRICHARDSON)
-                              ("bicgstab",   KSPBCGS);
+= { {"default",  ""},
+    {"cg",         KSPCG},
+    {"gmres",      KSPGMRES},
+    {"minres",     KSPMINRES},
+    {"tfqmr",      KSPTFQMR},
+    {"richardson", KSPRICHARDSON},
+    {"bicgstab",   KSPBCGS},
+    {"nash",       KSPNASH},
+    {"stcg",       KSPSTCG} };
 
 // Mapping from method string to description
-const std::vector<std::pair<std::string, std::string> >
-  PETScKrylovSolver::_methods_descr = boost::assign::pair_list_of
-    ("default",    "default Krylov method")
-    ("cg",         "Conjugate gradient method")
-    ("gmres",      "Generalized minimal residual method")
-    ("minres",     "Minimal residual method")
-    ("tfqmr",      "Transpose-free quasi-minimal residual method")
-    ("richardson", "Richardson method")
-    ("bicgstab",   "Biconjugate gradient stabilized method");
+const std::map<std::string, std::string>
+PETScKrylovSolver::_methods_descr
+=
+{ {"default",    "default Krylov method"},
+  {"cg",         "Conjugate gradient method"},
+  {"gmres",      "Generalized minimal residual method"},
+  {"minres",     "Minimal residual method"},
+  {"tfqmr",      "Transpose-free quasi-minimal residual method"},
+  {"richardson", "Richardson method"},
+  {"bicgstab",   "Biconjugate gradient stabilized method"} };
 
 //-----------------------------------------------------------------------------
-std::vector<std::pair<std::string, std::string> >
+std::map<std::string, std::string>
 PETScKrylovSolver::methods()
 {
   return PETScKrylovSolver::_methods_descr;
 }
 //-----------------------------------------------------------------------------
-std::vector<std::pair<std::string, std::string> >
+std::map<std::string, std::string>
 PETScKrylovSolver::preconditioners()
 {
   return PETScPreconditioner::preconditioners();
 }
 //-----------------------------------------------------------------------------
-void PETScKrylovSolver::set_options_prefix(std::string prefix)
-{
-  dolfin_assert(_ksp);
-  PetscErrorCode ierr = KSPSetOptionsPrefix(*_ksp, prefix.c_str());
-  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetOptionsPrefix");
-}
-//-----------------------------------------------------------------------------
 Parameters PETScKrylovSolver::default_parameters()
 {
   Parameters p(KrylovSolver::default_parameters());
   p.rename("petsc_krylov_solver");
 
-  // Norm type used in covergence test
+  // Norm type used in convergence test
   std::set<std::string> allowed_norm_types;
   allowed_norm_types.insert("preconditioned");
   allowed_norm_types.insert("true");
   allowed_norm_types.insert("none");
   p.add("convergence_norm_type", allowed_norm_types);
 
-  // Control PETSc performance profiling
-  p.add<bool>("profile");
-
   return p;
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method,
                                      std::string preconditioner)
-  : pc_dolfin(0), _preconditioner(new PETScPreconditioner(preconditioner)),
-    preconditioner_set(false)
+  : _ksp(NULL), pc_dolfin(NULL),
+    _preconditioner(new PETScPreconditioner(preconditioner)),
+    petsc_nullspace(NULL), preconditioner_set(false)
 {
   // Check that the requested method is known
   if (_methods.count(method) == 0)
@@ -152,8 +113,11 @@ PETScKrylovSolver::PETScKrylovSolver(std::string method,
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method,
                                      PETScPreconditioner& preconditioner)
- : _preconditioner(reference_to_no_delete_pointer(preconditioner)),
-   preconditioner_set(false)
+  : _ksp(NULL),
+    pc_dolfin(NULL),
+    _preconditioner(reference_to_no_delete_pointer(preconditioner)),
+    petsc_nullspace(NULL),
+    preconditioner_set(false)
 {
   // Set parameter values
   parameters = default_parameters();
@@ -162,9 +126,10 @@ PETScKrylovSolver::PETScKrylovSolver(std::string method,
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method,
-  boost::shared_ptr<PETScPreconditioner> preconditioner)
-  : _preconditioner(preconditioner), preconditioner_set(false)
-
+  std::shared_ptr<PETScPreconditioner> preconditioner)
+  : _ksp(NULL), pc_dolfin(NULL), _preconditioner(preconditioner),
+  petsc_nullspace(NULL),
+  preconditioner_set(false)
 {
   // Set parameter values
   parameters = default_parameters();
@@ -174,7 +139,8 @@ PETScKrylovSolver::PETScKrylovSolver(std::string method,
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method,
                                      PETScUserPreconditioner& preconditioner)
-  : pc_dolfin(&preconditioner), preconditioner_set(false)
+  : _ksp(NULL), pc_dolfin(&preconditioner), petsc_nullspace(NULL),
+    preconditioner_set(false)
 {
   // Set parameter values
   parameters = default_parameters();
@@ -183,8 +149,9 @@ PETScKrylovSolver::PETScKrylovSolver(std::string method,
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method,
-  boost::shared_ptr<PETScUserPreconditioner> preconditioner)
-  : pc_dolfin(preconditioner.get()), preconditioner_set(false)
+  std::shared_ptr<PETScUserPreconditioner> preconditioner)
+  : _ksp(NULL), pc_dolfin(preconditioner.get()), petsc_nullspace(NULL),
+    preconditioner_set(false)
 {
   // Set parameter values
   parameters = default_parameters();
@@ -192,46 +159,47 @@ PETScKrylovSolver::PETScKrylovSolver(std::string method,
   init(method);
 }
 //-----------------------------------------------------------------------------
-PETScKrylovSolver::PETScKrylovSolver(boost::shared_ptr<KSP> ksp)
-  : pc_dolfin(0), _ksp(ksp), preconditioner_set(true)
+PETScKrylovSolver::PETScKrylovSolver(KSP ksp)
+  : _ksp(ksp), pc_dolfin(0), petsc_nullspace(NULL), preconditioner_set(true)
 {
   // Set parameter values
   parameters = default_parameters();
+
+  // Increment reference count
+  if (_ksp)
+    PetscObjectReference((PetscObject)_ksp);
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::~PETScKrylovSolver()
 {
-  // Do nothing
+  if (_ksp)
+    KSPDestroy(&_ksp);
+  if (petsc_nullspace)
+    MatNullSpaceDestroy(&petsc_nullspace);
 }
 //-----------------------------------------------------------------------------
-void PETScKrylovSolver::set_operator(const boost::shared_ptr<const GenericLinearOperator> A)
+void
+PETScKrylovSolver::set_operator(std::shared_ptr<const GenericLinearOperator> A)
 {
   set_operators(A, A);
 }
 //-----------------------------------------------------------------------------
-void PETScKrylovSolver::set_operator(const boost::shared_ptr<const PETScBaseMatrix> A)
+void PETScKrylovSolver::set_operators(
+  std::shared_ptr<const  GenericLinearOperator> A,
+  std::shared_ptr<const GenericLinearOperator> P)
 {
-  set_operators(A, A);
-}
-//-----------------------------------------------------------------------------
-void PETScKrylovSolver::set_operators(const boost::shared_ptr<const  GenericLinearOperator> A,
-                                      const boost::shared_ptr<const GenericLinearOperator> P)
-{
-  set_operators(as_type<const PETScBaseMatrix>(A),
-                as_type<const PETScBaseMatrix>(P));
-}
-//-----------------------------------------------------------------------------
-void PETScKrylovSolver::set_operators(const boost::shared_ptr<const PETScBaseMatrix> A,
-                              const boost::shared_ptr<const PETScBaseMatrix> P)
-{
-  _A = A;
-  _P = P;
-  dolfin_assert(_A);
-  dolfin_assert(_P);
+  _set_operators(as_type<const PETScBaseMatrix>(A),
+                 as_type<const PETScBaseMatrix>(P));
 }
 //-----------------------------------------------------------------------------
 void PETScKrylovSolver::set_nullspace(const VectorSpaceBasis& nullspace)
 {
+#if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 5
+
+  deprecation("PETScKrylovSolver::set_nullspace",
+              "1.6.0", "1.7.0",
+              "Attach the null space to the matrix insetad");
+
   PetscErrorCode ierr;
 
   // Copy vectors
@@ -248,34 +216,40 @@ void PETScKrylovSolver::set_nullspace(const VectorSpaceBasis& nullspace)
   std::vector<Vec> petsc_vec(nullspace.dim());
   for (std::size_t i = 0; i < nullspace.dim(); ++i)
   {
-    petsc_vec[i] = *(_nullspace[i].vec().get());
+    petsc_vec[i] = _nullspace[i].vec();
     PetscReal val = 0.0;
-    ierr = VecNormalize(petsc_vec[i], &val);
+    ierr = VecNormalize(_nullspace[i].vec(), &val);
     if (ierr != 0) petsc_error(ierr, __FILE__, "VecNormalize");
   }
 
   // Create null space
-  petsc_nullspace.reset(new MatNullSpace, PETScMatNullSpaceDeleter());
-  ierr = MatNullSpaceCreate(PETSC_COMM_WORLD, PETSC_FALSE,
-                            nullspace.dim(),
-                            petsc_vec.data(), petsc_nullspace.get());
+  if (petsc_nullspace)
+    MatNullSpaceDestroy(&petsc_nullspace);
+  ierr = MatNullSpaceCreate(PETSC_COMM_WORLD, PETSC_FALSE, nullspace.dim(),
+                            petsc_vec.data(), &petsc_nullspace);
   if (ierr != 0) petsc_error(ierr, __FILE__, "MatNullSpaceCreate");
 
   // Set null space
   dolfin_assert(_ksp);
-  ierr = KSPSetNullSpace(*_ksp, *petsc_nullspace);
+  ierr = KSPSetNullSpace(_ksp, petsc_nullspace);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetNullSpace");
+
+#else
+  dolfin_error("PETScKrylovSolver.cpp",
+               "set null space for solver",
+               "For PETSc version > 3.5 nullspace must be attached to matrix");
+#endif
 }
 //-----------------------------------------------------------------------------
 const PETScBaseMatrix& PETScKrylovSolver::get_operator() const
 {
-  if (!_A)
+  if (!_matA)
   {
     dolfin_error("PETScKrylovSolver.cpp",
                  "access operator for PETSc Krylov solver",
                  "Operator has not been set");
   }
-  return *_A;
+  return *_matA;
 }
 //-----------------------------------------------------------------------------
 std::size_t PETScKrylovSolver::solve(GenericVector& x, const GenericVector& b)
@@ -284,43 +258,46 @@ std::size_t PETScKrylovSolver::solve(GenericVector& x, const GenericVector& b)
 }
 //-----------------------------------------------------------------------------
 std::size_t PETScKrylovSolver::solve(const GenericLinearOperator& A,
-                                      GenericVector& x,
-                                      const GenericVector& b)
+                                     GenericVector& x,
+                                     const GenericVector& b)
 {
-  return solve(as_type<const PETScBaseMatrix>(A),
-               as_type<PETScVector>(x),
-               as_type<const PETScVector>(b));
+  return _solve(as_type<const PETScBaseMatrix>(A),
+                as_type<PETScVector>(x),
+                as_type<const PETScVector>(b));
 }
 //-----------------------------------------------------------------------------
 std::size_t PETScKrylovSolver::solve(PETScVector& x, const PETScVector& b)
 {
   Timer timer("PETSc Krylov solver");
 
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
   dolfin_assert(_ksp);
 
   PetscErrorCode ierr;
 
   // Check dimensions
-  const std::size_t M = _A->size(0);
-  const std::size_t N = _A->size(1);
-  if (_A->size(0) != b.size())
+  const std::size_t M = _matA->size(0);
+  const std::size_t N = _matA->size(1);
+  if (_matA->size(0) != b.size())
   {
     dolfin_error("PETScKrylovSolver.cpp",
                  "unable to solve linear system with PETSc Krylov solver",
-                 "Non-matching dimensions for linear system (matrix has %d rows and right-hand side vector has %d rows)",
-                 _A->size(0), b.size());
+                 "Non-matching dimensions for linear system (matrix has %ld rows and right-hand side vector has %ld rows)",
+                 _matA->size(0), b.size());
   }
 
   // Write a message
   const bool report = parameters["report"];
-  if (report && dolfin::MPI::process_number() == 0)
-    info("Solving linear system of size %d x %d (PETSc Krylov solver).", M, N);
+  if (report && dolfin::MPI::rank(PETSC_COMM_WORLD) == 0)
+  {
+    info("Solving linear system of size %ld x %ld (PETSc Krylov solver).",
+         M, N);
+  }
 
   // Reinitialize solution vector if necessary
-  if (x.size() != M)
+  if (x.empty())
   {
-    _A->resize(x, 1);
+    _matA->init_vector(x, 1);
     x.zero();
   }
 
@@ -328,29 +305,22 @@ std::size_t PETScKrylovSolver::solve(PETScVector& x, const PETScVector& b)
   set_petsc_ksp_options();
 
   // Set operators
-  set_petsc_operators();
+  //set_petsc_operators();
 
-  // Set (approxinate) null space for preconditioner
+  // Set near null space for preconditioner
   if (_preconditioner)
   {
-    dolfin_assert(_P);
-    boost::shared_ptr<const MatNullSpace> pc_nullspace
-      = _preconditioner->near_nullspace();
+    dolfin_assert(_matP);
+    const MatNullSpace pc_nullspace = _preconditioner->near_nullspace();
 
     if (pc_nullspace && !preconditioner_set)
     {
-      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR >= 3
-      ierr = MatSetNearNullSpace(*(_P->mat()), *pc_nullspace);
+      ierr = MatSetNearNullSpace(_matP->mat(), pc_nullspace);
       if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetNearNullSpace");
-      #else
-      dolfin_error("PETScMatrix.cpp",
-                   "set approximate null space for PETSc matrix",
-                   "This is supported by PETSc version > 3.2");
-      #endif
     }
   }
 
-  // FIXME: Improve check for re-setting preconditoner, e.g. if
+  // FIXME: Improve check for re-setting preconditioner, e.g. if
   //        parameters change
   // FIXME: Solve using matrix free matrices fails if no user provided
   //        Prec is provided
@@ -363,17 +333,10 @@ std::size_t PETScKrylovSolver::solve(PETScVector& x, const PETScVector& b)
   // User defined preconditioner
   else if (pc_dolfin && !preconditioner_set)
   {
-    PETScUserPreconditioner::setup(*_ksp, *pc_dolfin);
+    PETScUserPreconditioner::setup(_ksp, *pc_dolfin);
     preconditioner_set = true;
   }
 
-  // Check whether we need a work-around for a bug in PETSc-stable.
-  // This has been fixed in PETSc-dev, see
-  // https://bugs.launchpad.net/dolfin/+bug/988494
-  const bool use_petsc_cusp_hack = parameters["use_petsc_cusp_hack"];
-  if (use_petsc_cusp_hack)
-    info("Using hack to get around PETScCusp bug: ||b|| = %g", b.norm("l2"));
-
   // Set convergence norm type
   if (parameters["convergence_norm_type"].is_set())
   {
@@ -381,59 +344,49 @@ std::size_t PETScKrylovSolver::solve(PETScVector& x, const PETScVector& b)
       = parameters["convergence_norm_type"];
     if (convergence_norm_type == "true")
     {
-      ierr = KSPSetNormType(*_ksp, KSP_NORM_UNPRECONDITIONED);
+      ierr = KSPSetNormType(_ksp, KSP_NORM_UNPRECONDITIONED);
       if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetNormType");
     }
     else if (convergence_norm_type == "preconditioned")
     {
-      ierr = KSPSetNormType(*_ksp, KSP_NORM_PRECONDITIONED);
+      ierr = KSPSetNormType(_ksp, KSP_NORM_PRECONDITIONED);
       if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetNormType");
     }
     else if (convergence_norm_type == "none")
     {
-      ierr = KSPSetNormType(*_ksp, KSP_NORM_NONE);
+      ierr = KSPSetNormType(_ksp, KSP_NORM_NONE);
       if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetNormType");
     }
   }
 
   // Solve linear system
-  if (MPI::process_number() == 0)
+  if (MPI::rank(PETSC_COMM_WORLD) == 0)
   {
     log(PROGRESS, "PETSc Krylov solver starting to solve %i x %i system.",
-        _A->size(0), _A->size(1));
+        _matA->size(0), _matA->size(1));
   }
 
-  if (parameters["profile"].is_set())
-  {
-    const bool profile_performance = parameters["profile"];
-    if (profile_performance)
-    {
-      PetscLogBegin();
-      ierr = KSPSolve(*_ksp, *b.vec(), *x.vec());
-      if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSolve");
-      PetscLogView(PETSC_VIEWER_STDOUT_WORLD);
-    }
-  }
-  else
-  {
-    ierr =  KSPSolve(*_ksp, *b.vec(), *x.vec());
-    if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSolve");
-  }
+  ierr =  KSPSolve(_ksp, b.vec(), x.vec());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSolve");
+
+  // Update ghost values
+  x.update_ghost_values();
 
   // Get the number of iterations
   PetscInt num_iterations = 0;
-  ierr = KSPGetIterationNumber(*_ksp, &num_iterations);
+  ierr = KSPGetIterationNumber(_ksp, &num_iterations);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGetIterationNumber");
 
-  // Check if the solution converged and print error/warning if not converged
+  // Check if the solution converged and print error/warning if not
+  // converged
   KSPConvergedReason reason;
-  ierr = KSPGetConvergedReason(*_ksp, &reason);
+  ierr = KSPGetConvergedReason(_ksp, &reason);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGetConvergedReason");
   if (reason < 0)
   {
     // Get solver residual norm
     double rnorm = 0.0;
-    ierr = KSPGetResidualNorm(*_ksp, &rnorm);
+    ierr = KSPGetResidualNorm(_ksp, &rnorm);
     if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGetResidualNorm");
     const char *reason_str = KSPConvergedReasons[reason];
     bool error_on_nonconvergence = parameters["error_on_nonconvergence"];
@@ -452,37 +405,65 @@ std::size_t PETScKrylovSolver::solve(PETScVector& x, const PETScVector& b)
   }
 
   // Report results
-  if (report && dolfin::MPI::process_number() == 0)
+  if (report && dolfin::MPI::rank(PETSC_COMM_WORLD) == 0)
     write_report(num_iterations, reason);
 
   return num_iterations;
 }
 //-----------------------------------------------------------------------------
-std::size_t PETScKrylovSolver::solve(const PETScBaseMatrix& A,
-                                      PETScVector& x,
-                                      const PETScVector& b)
+void PETScKrylovSolver::set_reuse_preconditioner(bool reuse_pc)
 {
-  // Set operator
-  boost::shared_ptr<const PETScBaseMatrix> Atmp(&A, NoDeleter());
-  set_operator(Atmp);
-
-  // Call solve
-  return solve(x, b);
+#if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR >= 5
+  dolfin_assert(_ksp);
+  const PetscBool _reuse_pc = reuse_pc ? PETSC_TRUE : PETSC_FALSE;
+  PetscErrorCode ierr = KSPSetReusePreconditioner(_ksp, _reuse_pc);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetReusePreconditioner");
+#else
+  warning("PETScKrylovSolver::set_reuse_preconditioner is only supported for PETSc version 3.5 or later");
+#endif
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<KSP> PETScKrylovSolver::ksp() const
+void PETScKrylovSolver::set_options_prefix(std::string options_prefix)
+{
+  if (_ksp)
+  {
+    dolfin_error("PETScKrylovSolver.cpp",
+                 "setting PETSc options prefix",
+                 "Cannot set options prefix since PETSc KSP has already been initialized");
+  }
+  else
+    _petsc_options_prefix = options_prefix;
+}
+//-----------------------------------------------------------------------------
+std::string PETScKrylovSolver::get_options_prefix() const
+{
+  if (_ksp)
+  {
+    const char* prefix = NULL;
+    KSPGetOptionsPrefix(_ksp, &prefix);
+    return std::string(prefix);
+  }
+  else
+  {
+    warning("PETSc KSP object has not been initialised, therefore prefix has not been set");
+    return std::string();
+  }
+}
+//-----------------------------------------------------------------------------
+KSP PETScKrylovSolver::ksp() const
 {
   return _ksp;
 }
 //-----------------------------------------------------------------------------
 std::string PETScKrylovSolver::str(bool verbose) const
 {
+  dolfin_assert(_ksp);
   std::stringstream s;
   if (verbose)
   {
     warning("Verbose output for PETScKrylovSolver not implemented, calling \
 PETSc KSPView directly.");
-    PetscErrorCode ierr = KSPView(*_ksp, PETSC_VIEWER_STDOUT_WORLD);
+    PetscErrorCode ierr = KSPView(_ksp, PETSC_VIEWER_STDOUT_WORLD);
     if (ierr != 0) petsc_error(ierr, __FILE__, "KSPView");
   }
   else
@@ -495,64 +476,66 @@ void PETScKrylovSolver::init(const std::string& method)
 {
   PetscErrorCode ierr;
 
-  // Check that nobody else shares this solver
-  if (_ksp && !_ksp.unique())
-  {
-    dolfin_error("PETScKrylovSolver.cpp",
-                 "initialize PETSc Krylov solver",
-                 "More than one object points to the underlying PETSc object");
-  }
-
-  // Create new KSP object
-  _ksp.reset(new KSP, PETScKSPDeleter());
+  if (_ksp)
+    KSPDestroy(&_ksp);
 
   // Set up solver environment
-  if (MPI::num_processes() > 1)
-  {
-    ierr = KSPCreate(PETSC_COMM_WORLD, _ksp.get());
-    if (ierr != 0) petsc_error(ierr, __FILE__, "KSPCreate");
-  }
-  else
-  {
-    ierr = KSPCreate(PETSC_COMM_SELF, _ksp.get());
-    if (ierr != 0) petsc_error(ierr, __FILE__, "KSPCreate");
-  }
+  ierr = KSPCreate(PETSC_COMM_WORLD, &_ksp);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPCreate");
 
-  // Set some options
-  ierr = KSPSetFromOptions(*_ksp);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetFromOptions");
+  // Set options prefix (if any)
+  ierr = KSPSetOptionsPrefix(_ksp, _petsc_options_prefix.c_str());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetOptionsPrefix");
 
   // Set solver type
   if (method != "default")
   {
-    ierr = KSPSetType(*_ksp, _methods.find(method)->second);
+    ierr = KSPSetType(_ksp, _methods.find(method)->second);
     if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetType");
   }
+
+  // Set from options database
+  ierr = KSPSetFromOptions(_ksp);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetFromOptions");
 }
 //-----------------------------------------------------------------------------
-void PETScKrylovSolver::set_petsc_operators()
+void PETScKrylovSolver::_set_operator(std::shared_ptr<const PETScBaseMatrix> A)
 {
-  dolfin_assert(_A);
-  dolfin_assert(_P);
+  _set_operators(A, A);
+}
+//-----------------------------------------------------------------------------
+void
+PETScKrylovSolver::_set_operators(std::shared_ptr<const PETScBaseMatrix> A,
+                                 std::shared_ptr<const PETScBaseMatrix> P)
+{
+  _matA = A;
+  _matP = P;
+  dolfin_assert(_matA);
+  dolfin_assert(_matP);
+  dolfin_assert(_ksp);
+
+  PetscErrorCode ierr;
 
   // Get parameter
+  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 4
   const std::string mat_structure = parameters("preconditioner")["structure"];
 
   // Set operators with appropriate option
-  PetscErrorCode ierr;
   if (mat_structure == "same")
   {
-    ierr = KSPSetOperators(*_ksp, *_A->mat(), *_P->mat(), SAME_PRECONDITIONER);
+    ierr = KSPSetOperators(_ksp, _matA->mat(), _matP->mat(),
+                           SAME_PRECONDITIONER);
     if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetOperators");
   }
   else if (mat_structure == "same_nonzero_pattern")
   {
-    ierr = KSPSetOperators(*_ksp, *_A->mat(), *_P->mat(), SAME_NONZERO_PATTERN);
+    ierr = KSPSetOperators(_ksp, _matA->mat(), _matP->mat(),
+                           SAME_NONZERO_PATTERN);
     if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetOperators");
   }
   else if (mat_structure == "different_nonzero_pattern")
   {
-    ierr = KSPSetOperators(*_ksp, *_A->mat(), *_P->mat(),
+    ierr = KSPSetOperators(_ksp, _matA->mat(), _matP->mat(),
                            DIFFERENT_NONZERO_PATTERN);
     if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetOperators");
   }
@@ -560,9 +543,25 @@ void PETScKrylovSolver::set_petsc_operators()
   {
     dolfin_error("PETScKrylovSolver.cpp",
                  "set PETSc Krylov solver operators",
-                 "Preconditioner re-use paramrter \"%s \" is unknown",
+                 "Preconditioner re-use parameter \"%s \" is unknown",
                  mat_structure.c_str());
   }
+  #else
+  ierr = KSPSetOperators(_ksp, _matA->mat(), _matP->mat());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetOperators");
+  #endif
+}
+//-----------------------------------------------------------------------------
+std::size_t PETScKrylovSolver::_solve(const PETScBaseMatrix& A,
+                                      PETScVector& x,
+                                      const PETScVector& b)
+{
+  // Set operator
+  std::shared_ptr<const PETScBaseMatrix> Atmp(&A, NoDeleter());
+  _set_operator(Atmp);
+
+  // Call solve
+  return solve(x, b);
 }
 //-----------------------------------------------------------------------------
 void PETScKrylovSolver::set_petsc_ksp_options()
@@ -571,7 +570,7 @@ void PETScKrylovSolver::set_petsc_ksp_options()
 
   // GMRES restart parameter
   const int gmres_restart = parameters("gmres")["restart"];
-  ierr = KSPGMRESSetRestart(*_ksp, gmres_restart);
+  ierr = KSPGMRESSetRestart(_ksp, gmres_restart);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGMRESSetRestart");
 
   // Non-zero initial guess
@@ -579,20 +578,20 @@ void PETScKrylovSolver::set_petsc_ksp_options()
   PetscBool petsc_nonzero_guess = PETSC_FALSE;
   if (nonzero_guess)
     petsc_nonzero_guess = PETSC_TRUE;
-  ierr = KSPSetInitialGuessNonzero(*_ksp, petsc_nonzero_guess);
+  ierr = KSPSetInitialGuessNonzero(_ksp, petsc_nonzero_guess);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetInitialGuessNonzero");
 
   // Monitor convergence
   const bool monitor_convergence = parameters["monitor_convergence"];
   if (monitor_convergence)
   {
-    ierr = KSPMonitorSet(*_ksp, KSPMonitorTrueResidualNorm, 0, 0);
+    ierr = KSPMonitorSet(_ksp, KSPMonitorTrueResidualNorm, 0, 0);
     if (ierr != 0) petsc_error(ierr, __FILE__, "KSPMonitorSet");
   }
 
   // Set tolerances
   const int max_iterations = parameters["maximum_iterations"];
-  ierr = KSPSetTolerances(*_ksp,
+  ierr = KSPSetTolerances(_ksp,
                           parameters["relative_tolerance"],
                           parameters["absolute_tolerance"],
                           parameters["divergence_limit"],
@@ -603,36 +602,29 @@ void PETScKrylovSolver::set_petsc_ksp_options()
 void PETScKrylovSolver::write_report(int num_iterations,
                                      KSPConvergedReason reason)
 {
+  dolfin_assert(_ksp);
+
   PetscErrorCode ierr;
 
   // Get name of solver and preconditioner
   PC pc;
-  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 3
-  const KSPType ksp_type;
-  const PCType pc_type;
-  #else
   KSPType ksp_type;
   PCType pc_type;
-  #endif
-  ierr = KSPGetType(*_ksp, &ksp_type);
+
+  ierr = KSPGetType(_ksp, &ksp_type);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGetType");
 
-  ierr = KSPGetPC(*_ksp, &pc);
+  ierr = KSPGetPC(_ksp, &pc);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGetPC");
 
   ierr = PCGetType(pc, &pc_type);
   if (ierr != 0) petsc_error(ierr, __FILE__, "PCGetType");
 
-  // If using additive Schwarz or block Jacobi, get 'sub' method which is
-  // applied to each block
+  // If using additive Schwarz or block Jacobi, get 'sub' method which
+  // is applied to each block
   const std::string pc_type_str = pc_type;
-  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 3
-  const KSPType sub_ksp_type;
-  const PCType sub_pc_type;
-  #else
   KSPType sub_ksp_type;
   PCType sub_pc_type;
-  #endif
   PC sub_pc;
   KSP* sub_ksp = NULL;
   if (pc_type_str == PCASM || pc_type_str == PCBJACOBI)
@@ -706,7 +698,7 @@ void PETScKrylovSolver::check_dimensions(const PETScBaseMatrix& A,
   {
     dolfin_error("PETScKrylovSolver.cpp",
                  "unable to solve linear system with PETSc Krylov solver",
-                 "Non-matching dimensions for linear system (matrix has %d rows and right-hand side vector has %d rows)",
+                 "Non-matching dimensions for linear system (matrix has %ld rows and right-hand side vector has %ld rows)",
                  A.size(0), b.size());
   }
 
@@ -715,7 +707,7 @@ void PETScKrylovSolver::check_dimensions(const PETScBaseMatrix& A,
   {
     dolfin_error("PETScKrylovSolver.cpp",
                  "unable to solve linear system with PETSc Krylov solver",
-                 "Non-matching dimensions for linear system (matrix has %d columns and solution vector has %d rows)",
+                 "Non-matching dimensions for linear system (matrix has %ld columns and solution vector has %ld rows)",
                  A.size(1), x.size());
   }
 
