@@ -27,7 +27,7 @@
 #include <memory>
 #include <dolfin/common/NoDeleter.h>
 #include <dolfin/common/types.h>
-#include <dolfin/log/dolfin_log.h>
+#include <dolfin/log/log.h>
 #include <dolfin/common/MPI.h>
 #include "PETScVector.h"
 #include "PETScLinearOperator.h"
@@ -135,7 +135,7 @@ void PETScLinearOperator::init_layout(const GenericVector& x,
                         (void*) this, &_matA);
   if (ierr != 0) petsc_error(ierr, __FILE__, "MatCreateShell");
 
-  // Incrase reference count
+  // Increase reference count
   PetscObjectReference((PetscObject)_matA);
 
   ierr = MatShellSetOperation(_matA, MATOP_MULT, (void (*)()) usermult);
