@@ -1,4 +1,4 @@
-// Copyright (C) 2005-2011 Garth N. Wells
+// Copyright (C) 2005-2014 Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -19,14 +19,11 @@
 // Modified by Anders Logg 2008-2012
 // Modified by Marie Rognes 2009
 // Modified by Fredrik Valdmanis 2011
-//
-// First added:  2005-08-31
-// Last changed: 2012-02-22
 
 #ifdef HAS_SLEPC
 
 #include <slepcversion.h>
-#include <dolfin/log/dolfin_log.h>
+#include <dolfin/log/log.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/NoDeleter.h>
 #include "PETScMatrix.h"
@@ -45,7 +42,7 @@ SLEPcEigenSolver::SLEPcEigenSolver(const PETScMatrix& A)
   parameters = default_parameters();
 
   // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &eps);
+  EPSCreate(PETSC_COMM_WORLD, &_eps);
 }
 //-----------------------------------------------------------------------------
 SLEPcEigenSolver::SLEPcEigenSolver(const PETScMatrix& A, const PETScMatrix& B)
@@ -59,7 +56,7 @@ SLEPcEigenSolver::SLEPcEigenSolver(const PETScMatrix& A, const PETScMatrix& B)
   parameters = default_parameters();
 
   // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &eps);
+  EPSCreate(PETSC_COMM_WORLD, &_eps);
 }
 //-----------------------------------------------------------------------------
 SLEPcEigenSolver::SLEPcEigenSolver(std::shared_ptr<const PETScMatrix> A)
@@ -71,7 +68,7 @@ SLEPcEigenSolver::SLEPcEigenSolver(std::shared_ptr<const PETScMatrix> A)
   parameters = default_parameters();
 
   // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &eps);
+  EPSCreate(PETSC_COMM_WORLD, &_eps);
 }
 //-----------------------------------------------------------------------------
 SLEPcEigenSolver::SLEPcEigenSolver(std::shared_ptr<const PETScMatrix> A,
@@ -87,14 +84,14 @@ SLEPcEigenSolver::SLEPcEigenSolver(std::shared_ptr<const PETScMatrix> A,
   parameters = default_parameters();
 
   // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &eps);
+  EPSCreate(PETSC_COMM_WORLD, &_eps);
 }
 //-----------------------------------------------------------------------------
 SLEPcEigenSolver::~SLEPcEigenSolver()
 {
   // Destroy solver environment
-  if (eps)
-    EPSDestroy(&eps);
+  if (_eps)
+    EPSDestroy(&_eps);
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::solve()
@@ -110,15 +107,16 @@ void SLEPcEigenSolver::solve(std::size_t n)
   dolfin_assert(_matA->size(0) == _matA->size(1));
   if (_matB)
   {
-    dolfin_assert(_matB->size(0) == _matB->size(1) && _matB->size(0) == _matA->size(0));
-    EPSSetOperators(eps, _matA->mat(), _matB->mat());
+    dolfin_assert(_matB->size(0) == _matB->size(1)
+                  && _matB->size(0) == _matA->size(0));
+    EPSSetOperators(_eps, _matA->mat(), _matB->mat());
   }
   else
-    EPSSetOperators(eps, _matA->mat(), PETSC_NULL);
+    EPSSetOperators(_eps, _matA->mat(), NULL);
 
   // Set number of eigenpairs to compute
   dolfin_assert(n <= _matA->size(0));
-  EPSSetDimensions(eps, n, PETSC_DECIDE, PETSC_DECIDE);
+  EPSSetDimensions(_eps, n, PETSC_DECIDE, PETSC_DECIDE);
 
   // Set parameters from local parameters
   read_parameters();
@@ -127,46 +125,42 @@ void SLEPcEigenSolver::solve(std::size_t n)
   std::string prefix = std::string(parameters["options_prefix"]);
   if (prefix != "default")
   {
-    // Make sure that the prefix has a '_' at the end if the user didn't provide it
+    // Make sure that the prefix has a '_' at the end if the user
+    // didn't provide it
     char lastchar = *prefix.rbegin();
     if (lastchar != '_')
       prefix += "_";
 
-    EPSSetOptionsPrefix(eps, prefix.c_str());
+    EPSSetOptionsPrefix(_eps, prefix.c_str());
   }
-  EPSSetFromOptions(eps);
+  EPSSetFromOptions(_eps);
 
   if (parameters["verbose"])
   {
     KSP ksp;
     ST st;
-    EPSMonitorSet(eps, EPSMonitorAll, PETSC_NULL, PETSC_NULL);
-    EPSSetType(eps, EPSARPACK);
-    EPSGetST(eps, &st);
+    EPSMonitorSet(_eps, EPSMonitorAll, NULL, NULL);
+    EPSGetST(_eps, &st);
     STGetKSP(st, &ksp);
-    KSPMonitorSet(ksp, KSPMonitorDefault, PETSC_NULL, PETSC_NULL);
-    EPSView(eps, PETSC_VIEWER_STDOUT_SELF);
+    KSPMonitorSet(ksp, KSPMonitorDefault, NULL, NULL);
+    EPSView(_eps, PETSC_VIEWER_STDOUT_SELF);
   }
 
   // Solve
-  EPSSolve(eps);
+  EPSSolve(_eps);
 
   // Check for convergence
   EPSConvergedReason reason;
-  EPSGetConvergedReason(eps, &reason);
+  EPSGetConvergedReason(_eps, &reason);
   if (reason < 0)
     warning("Eigenvalue solver did not converge");
 
   // Report solver status
   dolfin::la_index num_iterations = 0;
-  EPSGetIterationNumber(eps, &num_iterations);
+  EPSGetIterationNumber(_eps, &num_iterations);
 
-  #if SLEPC_VERSION_MAJOR == 3 && SLEPC_VERSION_MINOR < 4
-  const EPSType eps_type = NULL;
-  #else
   EPSType eps_type = NULL;
-  #endif
-  EPSGetType(eps, &eps_type);
+  EPSGetType(_eps, &eps_type);
   log(PROGRESS, "Eigenvalue solver (%s) converged in %d iterations.",
       eps_type, num_iterations);
 }
@@ -197,10 +191,10 @@ void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc,
 
   // Get number of computed values
   dolfin::la_index num_computed_eigenvalues;
-  EPSGetConverged(eps, &num_computed_eigenvalues);
+  EPSGetConverged(_eps, &num_computed_eigenvalues);
 
   if (ii < num_computed_eigenvalues)
-    EPSGetEigenvalue(eps, ii, &lr, &lc);
+    EPSGetEigenvalue(_eps, ii, &lr, &lc);
   else
   {
     dolfin_error("SLEPcEigenSolver.cpp",
@@ -226,7 +220,7 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
 
   // Get number of computed eigenvectors/values
   dolfin::la_index num_computed_eigenvalues;
-  EPSGetConverged(eps, &num_computed_eigenvalues);
+  EPSGetConverged(_eps, &num_computed_eigenvalues);
 
   if (ii < num_computed_eigenvalues)
   {
@@ -236,7 +230,7 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
 
     dolfin_assert(r.vec());
     dolfin_assert(c.vec());
-    EPSGetEigenpair(eps, ii, &lr, &lc, r.vec(), c.vec());
+    EPSGetEigenpair(_eps, ii, &lr, &lc, r.vec(), c.vec());
   }
   else
   {
@@ -249,14 +243,14 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
 std::size_t SLEPcEigenSolver::get_number_converged() const
 {
   dolfin::la_index num_conv;
-  EPSGetConverged(eps, &num_conv);
+  EPSGetConverged(_eps, &num_conv);
   return num_conv;
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::set_deflation_space(const PETScVector& deflation_space)
 {
   Vec x = deflation_space.vec();
-  EPSSetDeflationSpace(eps, 1, &x);
+  EPSSetDeflationSpace(_eps, 1, &x);
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::read_parameters()
@@ -277,15 +271,15 @@ void SLEPcEigenSolver::set_problem_type(std::string type)
     return;
 
   if (type == "hermitian")
-    EPSSetProblemType(eps, EPS_HEP);
+    EPSSetProblemType(_eps, EPS_HEP);
   else if (type == "non_hermitian")
-    EPSSetProblemType(eps, EPS_NHEP);
+    EPSSetProblemType(_eps, EPS_NHEP);
   else if (type == "gen_hermitian")
-    EPSSetProblemType(eps, EPS_GHEP);
+    EPSSetProblemType(_eps, EPS_GHEP);
   else if (type == "gen_non_hermitian")
-    EPSSetProblemType(eps, EPS_GNHEP);
+    EPSSetProblemType(_eps, EPS_GNHEP);
   else if (type == "pos_gen_non_hermitian")
-    EPSSetProblemType(eps, EPS_PGNHEP);
+    EPSSetProblemType(_eps, EPS_PGNHEP);
   else
   {
     dolfin_error("SLEPcEigenSolver.cpp",
@@ -301,7 +295,7 @@ void SLEPcEigenSolver::set_spectral_transform(std::string transform,
     return;
 
   ST st;
-  EPSGetST(eps, &st);
+  EPSGetST(_eps, &st);
   if (transform == "shift-and-invert")
   {
     STSetType(st, STSINVERT);
@@ -323,31 +317,31 @@ void SLEPcEigenSolver::set_spectrum(std::string spectrum)
 
   // Choose spectrum
   if (spectrum == "largest magnitude")
-    EPSSetWhichEigenpairs(eps, EPS_LARGEST_MAGNITUDE);
+    EPSSetWhichEigenpairs(_eps, EPS_LARGEST_MAGNITUDE);
   else if (spectrum == "smallest magnitude")
-    EPSSetWhichEigenpairs(eps, EPS_SMALLEST_MAGNITUDE);
+    EPSSetWhichEigenpairs(_eps, EPS_SMALLEST_MAGNITUDE);
   else if (spectrum == "largest real")
-    EPSSetWhichEigenpairs(eps, EPS_LARGEST_REAL);
+    EPSSetWhichEigenpairs(_eps, EPS_LARGEST_REAL);
   else if (spectrum == "smallest real")
-    EPSSetWhichEigenpairs(eps, EPS_SMALLEST_REAL);
+    EPSSetWhichEigenpairs(_eps, EPS_SMALLEST_REAL);
   else if (spectrum == "largest imaginary")
-    EPSSetWhichEigenpairs(eps, EPS_LARGEST_IMAGINARY);
+    EPSSetWhichEigenpairs(_eps, EPS_LARGEST_IMAGINARY);
   else if (spectrum == "smallest imaginary")
-    EPSSetWhichEigenpairs(eps, EPS_SMALLEST_IMAGINARY);
+    EPSSetWhichEigenpairs(_eps, EPS_SMALLEST_IMAGINARY);
   else if (spectrum == "target magnitude")
   {
-    EPSSetWhichEigenpairs(eps, EPS_TARGET_MAGNITUDE);
-    EPSSetTarget(eps, parameters["spectral_shift"]);
+    EPSSetWhichEigenpairs(_eps, EPS_TARGET_MAGNITUDE);
+    EPSSetTarget(_eps, parameters["spectral_shift"]);
   }
   else if (spectrum == "target real")
   {
-    EPSSetWhichEigenpairs(eps, EPS_TARGET_REAL);
-    EPSSetTarget(eps, parameters["spectral_shift"]);
+    EPSSetWhichEigenpairs(_eps, EPS_TARGET_REAL);
+    EPSSetTarget(_eps, parameters["spectral_shift"]);
   }
   else if (spectrum == "target imaginary")
   {
-    EPSSetWhichEigenpairs(eps, EPS_TARGET_IMAGINARY);
-    EPSSetTarget(eps, parameters["spectral_shift"]);
+    EPSSetWhichEigenpairs(_eps, EPS_TARGET_IMAGINARY);
+    EPSSetTarget(_eps, parameters["spectral_shift"]);
   }
   else
   {
@@ -367,23 +361,22 @@ void SLEPcEigenSolver::set_solver(std::string solver)
     return;
 
   // Choose solver
-
   // (Note that lanczos will give PETSc error unless problem_type is
   // set to 'hermitian' or 'gen_hermitian')
   if (solver == "power")
-    EPSSetType(eps, EPSPOWER);
+    EPSSetType(_eps, EPSPOWER);
   else if (solver == "subspace")
-    EPSSetType(eps, EPSSUBSPACE);
+    EPSSetType(_eps, EPSSUBSPACE);
   else if (solver == "arnoldi")
-    EPSSetType(eps, EPSARNOLDI);
+    EPSSetType(_eps, EPSARNOLDI);
   else if (solver == "lanczos")
-    EPSSetType(eps, EPSLANCZOS);
+    EPSSetType(_eps, EPSLANCZOS);
   else if (solver == "krylov-schur")
-    EPSSetType(eps, EPSKRYLOVSCHUR);
+    EPSSetType(_eps, EPSKRYLOVSCHUR);
   else if (solver == "lapack")
-    EPSSetType(eps, EPSLAPACK);
+    EPSSetType(_eps, EPSLAPACK);
   else if (solver == "arpack")
-    EPSSetType(eps, EPSARPACK);
+    EPSSetType(_eps, EPSARPACK);
   else
   {
     dolfin_error("SLEPcEigenSolver.cpp",
@@ -395,15 +388,20 @@ void SLEPcEigenSolver::set_solver(std::string solver)
 void SLEPcEigenSolver::set_tolerance(double tolerance, std::size_t maxiter)
 {
   dolfin_assert(tolerance > 0.0);
-  EPSSetTolerances(eps, tolerance, maxiter);
+  EPSSetTolerances(_eps, tolerance, maxiter);
 }
 //-----------------------------------------------------------------------------
 std::size_t SLEPcEigenSolver::get_iteration_number() const
 {
   dolfin::la_index num_iter;
-  EPSGetIterationNumber(eps, &num_iter);
+  EPSGetIterationNumber(_eps, &num_iter);
   return num_iter;
 }
 //-----------------------------------------------------------------------------
+EPS SLEPcEigenSolver::eps() const
+{
+  return _eps;
+}
+//-----------------------------------------------------------------------------
 
 #endif
