@@ -41,9 +41,9 @@ SparsityPattern::SparsityPattern(std::size_t primary_dim)
 SparsityPattern::SparsityPattern(
   const MPI_Comm mpi_comm,
   const std::vector<std::size_t>& dims,
-  const std::vector<std::pair<std::size_t, std::size_t> >& local_range,
-  const std::vector<const std::vector<std::size_t>* > local_to_global,
-  const std::vector<const std::vector<int>* > off_process_owner,
+  const std::vector<std::pair<std::size_t, std::size_t>>& local_range,
+  const std::vector<ArrayView<const std::size_t>>& local_to_global,
+  const std::vector<ArrayView<const int>>& off_process_owner,
   const std::vector<std::size_t>& block_sizes,
   std::size_t primary_dim)
   : GenericSparsityPattern(primary_dim), _mpi_comm(MPI_COMM_NULL)
@@ -55,9 +55,9 @@ SparsityPattern::SparsityPattern(
 void SparsityPattern::init(
   const MPI_Comm mpi_comm,
   const std::vector<std::size_t>& dims,
-  const std::vector<std::pair<std::size_t, std::size_t> >& local_range,
-  const std::vector<const std::vector<std::size_t>* > local_to_global,
-  const std::vector<const std::vector<int>* > off_process_owner,
+  const std::vector<std::pair<std::size_t, std::size_t>>& local_range,
+  const std::vector<ArrayView<const std::size_t>>& local_to_global,
+  const std::vector<ArrayView<const int>>& off_process_owner,
   const std::vector<std::size_t>& block_sizes)
 {
   // Only rank 2 sparsity patterns are supported
@@ -85,14 +85,16 @@ void SparsityPattern::init(
   // Set ownership range
   _local_range = local_range;
 
-  // Store copy of nonlocal index to owning process map
+  // Store copy of local-to-global and nonlocal index to owning
+  // process map
+  _local_to_global.resize(off_process_owner.size());
+  _off_process_owner.resize(off_process_owner.size());
   for (std::size_t i = 0; i < off_process_owner.size(); ++i)
   {
-    dolfin_assert(local_to_global[i]);
-    _local_to_global.push_back(*local_to_global[i]);
-
-    dolfin_assert(off_process_owner[i]);
-    _off_process_owner.push_back(*off_process_owner[i]);
+    _local_to_global[i].assign(local_to_global[i].begin(),
+                               local_to_global[i].end());
+    _off_process_owner[i].assign(off_process_owner[i].begin(),
+                                 off_process_owner[i].end());
   }
 
   // Check that primary dimension is valid
@@ -115,16 +117,14 @@ void SparsityPattern::init(
 }
 //-----------------------------------------------------------------------------
 void SparsityPattern::insert_global(
-  const std::vector<const std::vector<dolfin::la_index>* >& entries)
+  const std::vector<ArrayView<const dolfin::la_index>>& entries)
 {
   dolfin_assert(entries.size() == 2);
-  dolfin_assert(entries[0]);
-  dolfin_assert(entries[1]);
 
   const std::size_t _primary_dim = primary_dim();
 
-  const std::vector<dolfin::la_index>* map_i;
-  const std::vector<dolfin::la_index>* map_j;
+  ArrayView<const dolfin::la_index> map_i;
+  ArrayView<const dolfin::la_index> map_j;
   std::size_t primary_codim;
   dolfin_assert(_primary_dim < 2);
   if (_primary_dim == 0)
@@ -151,15 +151,13 @@ void SparsityPattern::insert_global(
   if (MPI::size(_mpi_comm) == 1)
   {
     // Sequential mode, do simple insertion
-    std::vector<dolfin::la_index>::const_iterator i_index;
-    for (i_index = map_i->begin(); i_index != map_i->end(); ++i_index)
-      diagonal[*i_index].insert(map_j->begin(), map_j->end());
+    for (auto i_index = map_i.begin(); i_index != map_i.end(); ++i_index)
+      diagonal[*i_index].insert(map_j.begin(), map_j.end());
   }
   else
   {
     // Parallel mode, use either diagonal, off_diagonal or non_local
-    std::vector<dolfin::la_index>::const_iterator i_index;
-    for (i_index = map_i->begin(); i_index != map_i->end(); ++i_index)
+    for (auto i_index = map_i.begin(); i_index != map_i.end(); ++i_index)
     {
       if (local_range0.first <= *i_index && *i_index < local_range0.second)
       {
@@ -167,8 +165,7 @@ void SparsityPattern::insert_global(
         const std::size_t I = *i_index - local_range0.first;
 
         // Store local entry in diagonal or off-diagonal block
-        std::vector<dolfin::la_index>::const_iterator j_index;
-        for (j_index = map_j->begin(); j_index != map_j->end(); ++j_index)
+        for (auto j_index = map_j.begin(); j_index != map_j.end(); ++j_index)
         {
           if (local_range1.first <= *j_index && *j_index < local_range1.second)
           {
@@ -191,16 +188,13 @@ void SparsityPattern::insert_global(
 }
 //-----------------------------------------------------------------------------
 void SparsityPattern::insert_local(
-  const std::vector<const std::vector<dolfin::la_index>* >& entries)
+  const std::vector<ArrayView<const dolfin::la_index>>& entries)
 {
   dolfin_assert(entries.size() == 2);
-  dolfin_assert(entries[0]);
-  dolfin_assert(entries[1]);
-
   const std::size_t _primary_dim = primary_dim();
 
-  const std::vector<dolfin::la_index>* map_i;
-  const std::vector<dolfin::la_index>* map_j;
+  ArrayView<const dolfin::la_index> map_i;
+  ArrayView<const dolfin::la_index> map_j;
   std::size_t primary_codim;
   dolfin_assert(_primary_dim < 2);
   if (_primary_dim == 0)
@@ -226,22 +220,19 @@ void SparsityPattern::insert_local(
   if (MPI::size(_mpi_comm) == 1)
   {
     // Sequential mode, do simple insertion
-    std::vector<dolfin::la_index>::const_iterator i_index;
-    for (i_index = map_i->begin(); i_index != map_i->end(); ++i_index)
-      diagonal[*i_index].insert(map_j->begin(), map_j->end());
+    for (auto i_index = map_i.begin(); i_index != map_i.end(); ++i_index)
+      diagonal[*i_index].insert(map_j.begin(), map_j.end());
   }
   else
   {
     // Parallel mode, use either diagonal, off_diagonal or non_local
-    std::vector<dolfin::la_index>::const_iterator i_index;
     std::size_t codim_block_size = _block_size[primary_codim];
-    for (i_index = map_i->begin(); i_index != map_i->end(); ++i_index)
+    for (auto i_index = map_i.begin(); i_index != map_i.end(); ++i_index)
     {
       if (*i_index < local_size0)
       {
         // Store local entry in diagonal or off-diagonal block
-        std::vector<dolfin::la_index>::const_iterator j_index;
-        for (j_index = map_j->begin(); j_index != map_j->end(); ++j_index)
+        for (auto j_index = map_j.begin(); j_index != map_j.end(); ++j_index)
         {
           if (*j_index < local_size1)
           {
@@ -265,9 +256,8 @@ void SparsityPattern::insert_local(
       else
       {
         // Store non-local entry (communicated later during apply())
-        std::vector<dolfin::la_index>::const_iterator j_index;
         std::size_t codim_block_size = _block_size[primary_codim];
-        for (j_index = map_j->begin(); j_index != map_j->end(); ++j_index)
+        for (auto j_index = map_j.begin(); j_index != map_j.end(); ++j_index)
         {
           // Get global index
           std::size_t J = 0;
@@ -275,8 +265,8 @@ void SparsityPattern::insert_local(
             J = *j_index + offset1;
           else
           {
-            const std::div_t div
-              = std::div((int) (*j_index - local_size1), (int) codim_block_size);
+            const std::div_t div = std::div((int) (*j_index - local_size1),
+                                            (int) codim_block_size);
             const int j_node = div.quot;
             const int j_component = div.rem;
 
@@ -308,25 +298,21 @@ std::pair<std::size_t, std::size_t>
 std::size_t SparsityPattern::num_nonzeros() const
 {
   std::size_t nz = 0;
-  typedef std::vector<set_type>::const_iterator slice_it;
-  for (slice_it slice = diagonal.begin(); slice != diagonal.end(); ++slice)
+  for (auto slice = diagonal.begin(); slice != diagonal.end(); ++slice)
     nz += slice->size();
-  for (slice_it slice = off_diagonal.begin(); slice != off_diagonal.end();
-       ++slice)
-  {
+  for (auto slice = off_diagonal.begin(); slice != off_diagonal.end(); ++slice)
     nz += slice->size();
-  }
+
   return nz;
 }
 //-----------------------------------------------------------------------------
-void  SparsityPattern::num_nonzeros_diagonal(std::vector<std::size_t>& num_nonzeros) const
+void SparsityPattern::num_nonzeros_diagonal(std::vector<std::size_t>& num_nonzeros) const
 {
   // Resize vector
   num_nonzeros.resize(diagonal.size());
 
   // Get number of nonzeros per generalised row
-  typedef std::vector<set_type>::const_iterator slice_it;
-  for (slice_it slice = diagonal.begin(); slice != diagonal.end(); ++slice)
+  for (auto slice = diagonal.begin(); slice != diagonal.end(); ++slice)
     num_nonzeros[slice - diagonal.begin()] = slice->size();
 }
 //-----------------------------------------------------------------------------
@@ -336,12 +322,8 @@ void SparsityPattern::num_nonzeros_off_diagonal(std::vector<std::size_t>& num_no
   num_nonzeros.resize(off_diagonal.size());
 
   // Compute number of nonzeros per generalised row
-  typedef std::vector<set_type>::const_iterator slice_it;
-  for (slice_it slice = off_diagonal.begin(); slice != off_diagonal.end();
-       ++slice)
-  {
+  for (auto slice = off_diagonal.begin(); slice != off_diagonal.end(); ++slice)
     num_nonzeros[slice - off_diagonal.begin()] = slice->size();
-  }
 }
 //-----------------------------------------------------------------------------
 void SparsityPattern::num_local_nonzeros(std::vector<std::size_t>& num_nonzeros) const
@@ -390,7 +372,7 @@ void SparsityPattern::apply()
   {
     // Figure out correct process for each non-local entry
     dolfin_assert(non_local.size() % 2 == 0);
-    std::vector<std::vector<std::size_t> > non_local_send(num_processes);
+    std::vector<std::vector<std::size_t>> non_local_send(num_processes);
 
     std::size_t dim_block_size = _block_size[_primary_dim];
     for (std::size_t i = 0; i < non_local.size(); i += 2)
@@ -429,7 +411,7 @@ void SparsityPattern::apply()
     }
 
     // Communicate non-local entries to other processes
-    std::vector<std::vector<std::size_t> > non_local_received;
+    std::vector<std::vector<std::size_t>> non_local_received;
     MPI::all_to_all(_mpi_comm, non_local_send, non_local_received);
 
     // Insert non-local entries received from other processes
@@ -483,7 +465,6 @@ std::string SparsityPattern::str(bool verbose) const
 {
   // Print each row
   std::stringstream s;
-  typedef set_type::const_iterator entry_it;
   for (std::size_t i = 0; i < diagonal.size(); i++)
   {
     if (primary_dim() == 0)
@@ -491,21 +472,18 @@ std::string SparsityPattern::str(bool verbose) const
     else
       s << "Col " << i << ":";
 
-    for (entry_it entry = diagonal[i].begin(); entry != diagonal[i].end();
-         ++entry)
-    {
+    for (auto entry = diagonal[i].begin(); entry != diagonal[i].end(); ++entry)
       s << " " << *entry;
-    }
     s << std::endl;
   }
 
   return s.str();
 }
 //-----------------------------------------------------------------------------
-std::vector<std::vector<std::size_t> >
+std::vector<std::vector<std::size_t>>
 SparsityPattern::diagonal_pattern(Type type) const
 {
-  std::vector<std::vector<std::size_t> > v(diagonal.size());
+  std::vector<std::vector<std::size_t>> v(diagonal.size());
   for (std::size_t i = 0; i < diagonal.size(); ++i)
     v[i].insert(v[i].begin(), diagonal[i].begin(), diagonal[i].end());
 
@@ -518,10 +496,10 @@ SparsityPattern::diagonal_pattern(Type type) const
   return v;
 }
 //-----------------------------------------------------------------------------
-std::vector<std::vector<std::size_t> >
+std::vector<std::vector<std::size_t>>
   SparsityPattern::off_diagonal_pattern(Type type) const
 {
-  std::vector<std::vector<std::size_t> > v(off_diagonal.size());
+  std::vector<std::vector<std::size_t>> v(off_diagonal.size());
   for (std::size_t i = 0; i < off_diagonal.size(); ++i)
     v[i].insert(v[i].begin(), off_diagonal[i].begin(), off_diagonal[i].end());
 
