@@ -79,13 +79,16 @@ std::size_t IntervalCell::orientation(const Cell& cell) const
   return cell.orientation(up);
 }
 //-----------------------------------------------------------------------------
-void IntervalCell::create_entities(std::vector<std::vector<unsigned int> >& e,
+void IntervalCell::create_entities(boost::multi_array<unsigned int, 2>& e,
                                    std::size_t dim, const unsigned int* v) const
 {
-  // We don't need to create any entities
-  dolfin_error("IntervalCell.cpp",
-               "create entities of interval cell",
-               "Don't know how to create entities of topological dimension %d", dim);
+  // For completeness, IntervalCell has two 'edges'
+  dolfin_assert(dim == 0);
+
+  // Resize data structure
+  e.resize(boost::extents[2][1]);
+  // Create the three edges
+  e[0][0] = v[0]; e[1][0] = v[1];
 }
 //-----------------------------------------------------------------------------
 void IntervalCell::refine_cell(Cell& cell, MeshEditor& editor,
@@ -192,7 +195,8 @@ double IntervalCell::squared_distance(const Point& point,
   return std::max(v0.dot(v0) - a0*a0 / v01.dot(v01), 0.0);
 }
 //-----------------------------------------------------------------------------
-double IntervalCell::normal(const Cell& cell, std::size_t facet, std::size_t i) const
+double IntervalCell::normal(const Cell& cell, std::size_t facet,
+                            std::size_t i) const
 {
   return normal(cell, facet)[i];
 }
@@ -251,7 +255,8 @@ double IntervalCell::facet_area(const Cell& cell, std::size_t facet) const
 }
 //-----------------------------------------------------------------------------
 void IntervalCell::order(Cell& cell,
-                 const std::vector<std::size_t>& local_to_global_vertex_indices) const
+                         const std::vector<std::size_t>&
+                         local_to_global_vertex_indices) const
 {
   // Sort i - j for i > j: 1 - 0
 
