@@ -15,11 +15,12 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
-// Modified by Anders Logg, 2008.
-// Modified by Kristoffer Sleim, 2008.
+// Modified by Anders Logg 2008-2014
+// Modified by Kristoffer Sleim 2008
+// Modified by August Johansson 2014
 //
 // First added:  2007-12-12
-// Last changed: 2013-08-26
+// Last changed: 2014-02-13
 
 #include <dolfin/log/log.h>
 #include "Cell.h"
@@ -75,7 +76,7 @@ std::size_t PointCell::orientation(const Cell& cell) const
   return 0;
 }
 //-----------------------------------------------------------------------------
-void PointCell::create_entities(std::vector<std::vector<std::size_t> >& e,
+void PointCell::create_entities(boost::multi_array<unsigned int, 2>& e,
                                 std::size_t dim,
                                 const unsigned int* v) const
 {
@@ -85,7 +86,7 @@ void PointCell::create_entities(std::vector<std::vector<std::size_t> >& e,
 }
 //-----------------------------------------------------------------------------
 void PointCell::refine_cell(Cell& cell, MeshEditor& editor,
-                          std::size_t& current_cell) const
+                            std::size_t& current_cell) const
 {
   dolfin_error("PointCell.cpp",
                "refine cell",
@@ -114,7 +115,8 @@ double PointCell::squared_distance(const Cell& cell, const Point& point) const
   return 0.0;
 }
 //-----------------------------------------------------------------------------
-double PointCell::normal(const Cell& cell, std::size_t facet, std::size_t i) const
+double PointCell::normal(const Cell& cell, std::size_t facet,
+                         std::size_t i) const
 {
   dolfin_error("PointCell.cpp",
                "find component of normal vector of cell",
@@ -148,8 +150,9 @@ double PointCell::facet_area(const Cell& cell, std::size_t facet) const
   return 0.0;
 }
 //-----------------------------------------------------------------------------
-void PointCell::order(Cell& cell,
-                 const std::vector<std::size_t>& local_to_global_vertex_indices) const
+void PointCell::order(
+  Cell& cell,
+  const std::vector<std::size_t>& local_to_global_vertex_indices) const
 {
   dolfin_error("PointCell.cpp",
                "order cell",
@@ -158,22 +161,18 @@ void PointCell::order(Cell& cell,
 //-----------------------------------------------------------------------------
 bool PointCell::collides(const Cell& cell, const Point& point) const
 {
-  dolfin_error("PointCell.cpp",
-               "check whether point collides with cell.",
-               "Not defined for point cells");
-
-  return false;
+  return CollisionDetection::collides(cell, point);
 }
 //-----------------------------------------------------------------------------
 bool PointCell::collides(const Cell& cell, const MeshEntity& entity) const
 {
-  dolfin_error("PointCell.cpp",
-               "check whether entity collides with cell.",
-               "Not defined for point cells");
-
-  dolfin_not_implemented();
-
-  return false;
+  return CollisionDetection::collides(cell, entity);
+}
+//-----------------------------------------------------------------------------
+std::vector<double>
+PointCell::triangulate_intersection(const Cell& c0, const Cell& c1) const
+{
+  return IntersectionTriangulation::triangulate_intersection(c0, c1);
 }
 //-----------------------------------------------------------------------------
 std::string PointCell::description(bool plural) const
