@@ -1,4 +1,3 @@
-
 // Copyright (C) 2012 Patrick E. Farrell
 //
 // This file is part of DOLFIN.
@@ -17,9 +16,6 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // Modified by Corrado Maurini, 2013.
-//
-// First added:  2012-10-13
-// Last changed: 2013-11-21
 
 #ifndef __PETSC_SNES_SOLVER_H
 #define __PETSC_SNES_SOLVER_H
@@ -28,20 +24,22 @@
 
 #include <map>
 #include <petscsnes.h>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include <dolfin/nls/NewtonSolver.h>
 #include <dolfin/parameter/Parameters.h>
 #include <dolfin/la/PETScObject.h>
+#include <dolfin/la/PETScVector.h>
 
 namespace dolfin
 {
+
   /// Forward declarations
   class PETScVector;
 
-  /// This class implements methods for solving nonlinear systems
-  /// via PETSc's SNES interface. It includes line search and trust
-  /// region techniques for globalising the convergence of the
-  /// nonlinear iteration.
+  /// This class implements methods for solving nonlinear systems via
+  /// PETSc's SNES interface. It includes line search and trust region
+  /// techniques for globalising the convergence of the nonlinear
+  /// iteration.
 
   class PETScSNESSolver : public PETScObject
   {
@@ -90,6 +88,10 @@ namespace dolfin
     std::pair<std::size_t, bool> solve(NonlinearProblem& nonlinear_function,
                                        GenericVector& x);
 
+    /// Set up the SNES object, but don't do anything yet, in case the
+    /// user wants to access the SNES object directly
+    void init(NonlinearProblem& nonlinear_problem, GenericVector& x);
+
     /// Return a list of available solver methods
     static std::vector<std::pair<std::string, std::string> > methods();
 
@@ -99,46 +101,77 @@ namespace dolfin
     Parameters parameters;
 
     /// Return PETSc SNES pointer
-    boost::shared_ptr<SNES> snes() const
+    SNES snes() const
     { return _snes; }
 
   private:
 
-    /// PETSc solver pointer
-    boost::shared_ptr<SNES> _snes;
+    struct snes_ctx_t
+    {
+      // Constructor
+      snes_ctx_t() : nonlinear_problem(NULL), x(NULL), f_tmp(NULL), xl(NULL),
+                     xu(NULL) {}
 
-    /// Initialize SNES solver
+      // Destructor
+      ~snes_ctx_t()
+      {
+        if (f_tmp)
+          VecDestroy(&f_tmp);
+      }
+
+      NonlinearProblem* nonlinear_problem;
+      PETScVector* x;
+      Vec f_tmp;
+      const PETScVector* xl;
+      const PETScVector* xu;
+    };
+
+    // PETSc solver pointer
+    SNES _snes;
+
+    // Initialize SNES solver
     void init(const std::string& method);
 
-    /// Update the linear solver parameters
+    // Update the linear solver parameters
     void set_linear_solver_parameters();
 
-    /// Available solvers
+    // Available solvers
     static const std::map<std::string,
       std::pair<std::string, const SNESType> > _methods;
 
-    /// The callback for PETSc to compute F, the nonlinear residual
+    // The callback for PETSc to compute F, the nonlinear residual
     static PetscErrorCode FormFunction(SNES snes, Vec x, Vec f, void* ctx);
 
-    /// The callback for PETSc to compute A, the Jacobian
+    // The callback for PETSc to compute A, the Jacobian
+    #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 4
     static PetscErrorCode FormJacobian(SNES snes, Vec x, Mat* A, Mat* B,
                                        MatStructure* flag, void* ctx);
+    #else
+    static PetscErrorCode FormJacobian(SNES snes, Vec x, Mat A, Mat B,
+                                       void* ctx);
+    #endif
 
-    /// Set the bounds on the problem from the parameters, if desired
-    /// Here, x is passed in as a model vector from which we make our Vecs
-    /// that tell PETSc the bounds if the "sign" parameter is used.
+    static PetscErrorCode FormObjective(SNES snes, Vec x, PetscReal* out,
+                                        void* ctx);
+
+    // Set the bounds on the problem from the parameters, if desired
+    // Here, x is passed in as a model vector from which we make our
+    // Vecs that tell PETSc the bounds if the "sign" parameter is
+    // used.
     void set_bounds(GenericVector& x);
 
     // Check if the problem is a variational inequality
     bool is_vi() const;
 
     // Upper and lower bounds for bound-constrained solvers
-    boost::shared_ptr<const PETScVector> lb;
-    boost::shared_ptr<const PETScVector> ub;
+    std::shared_ptr<const PETScVector> lb;
+    std::shared_ptr<const PETScVector> ub;
 
     // Flag to indicate if explicit bounds are set
     bool has_explicit_bounds;
 
+    // SNES context
+    struct snes_ctx_t _snes_ctx;
   };
 
 }
