@@ -158,7 +158,7 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
 
     for i, c in enumerate(expr):
         # Find groups of connected alphanumeric letters
-        symbols = re.findall(r"([a-zA-Z_]*:{0,2}[\w]+)", c)
+        symbols = re.findall(r"(?:[a-zA-Z_]*::)*[\w\.]+", c)
         assert isinstance(symbols, list)
         variables.update(symbols)
         for sym in symbols:
@@ -184,6 +184,9 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
     numerals = [v for v in variables if v[0] in "0123456789"]
     variables.difference_update(numerals)
 
+    # Remove bool literals
+    variables.difference_update(["true", "false"])
+
     # Remove any exponential representation
     exponentials = [v for v in variables if bool(re.search("e[0-9,\-,\.,\+]+", v))]
     variables.difference_update(exponentials+["e"])
@@ -215,7 +218,7 @@ def expect_list_of(argtype, arg, argname):
     if arg is None:
         return []
 
-    if isinstance(arg, (types.NoneType, list, tuple)):
+    if isinstance(arg, (list, tuple)):
         if all(isinstance(s, argtype) for s in arg):
             return arg
 
@@ -249,6 +252,7 @@ def check_swig_version(compiled_module):
 def compile_extension_module(code, module_name="",
                              additional_declarations="",
                              additional_system_headers=None,
+                             mpi_comm=None,
                              **instant_kwargs):
     """
     Just In Time compile DOLFIN C++ code into a Python module.
@@ -272,6 +276,11 @@ def compile_extension_module(code, module_name="",
         additional_system_headers :
             System headers needed to compile the generated can be included
             using this argument. The headers are passed using a list of 'str'
+            
+        mpi_comm
+            An mpi communicator used to determine what processors
+            should compile the module. By default mpi_world_comm is
+            used, but could also accept mpi_group communicators.
 
     *Returns*
 
@@ -332,16 +341,19 @@ def compile_extension_module(code, module_name="",
     # Create unique module name for this application run
     if module_name is "":
         module_name = "dolfin_compile_code_%s" % \
-                      hashlib.md5(repr(code) + dolfin.__version__ + \
-                                  str(_interface_version)+\
-                                  additional_declarations +\
-                                  str(additional_system_headers)).hexdigest()
+                      hashlib.sha1((repr(code) +
+                                   dolfin.__version__ +
+                                   str(_interface_version) +
+                                   additional_declarations +
+                                   str(additional_system_headers) +
+                                   repr(instant_kwargs)).encode("utf-8")
+                                   ).hexdigest()
 
     # Extract dolfin dependencies and class names
     used_types, declared_types = parse_and_extract_type_info(code)
 
     # Add any bases of the declared types to used_types
-    for declared_type, bases in declared_types.items():
+    for declared_type, bases in list(declared_types.items()):
         used_types.update(bases)
 
     # Filter out dolfin types and add derived and bases for each type
@@ -373,12 +385,15 @@ def compile_extension_module(code, module_name="",
         for dolfin_type in ["FunctionSpace", "Function"]:
             dependencies["function"].append(dolfin_type_def[dolfin_type]["header"])
 
-    # Add uint type
-    if "common" in dependencies:
-        dependencies["common"].append("dolfin/common/types.h")
-    else:
-        dependencies["common"] = ["dolfin/common/types.h"]
+    # Add dependency for constants and types
+    if "common" not in dependencies:
+        dependencies["common"] = ["dolfin/common/constants.h", "dolfin/common/types.h"]
 
+    else:
+        for f in ["dolfin/common/constants.h", "dolfin/common/types.h"]:
+            if f not in dependencies["common"]:
+                dependencies["common"].append(f)
+        
     # Sort the dependencies
     dependencies = sort_submodule_dependencies(dependencies, submodule_info)
 
@@ -463,7 +478,7 @@ def extract_shared_ptr_declaration(declared_types, used_dolfin_types, shared_ptr
         if dolfin_type in shared_ptr_classes:
             used_shared_ptr_types.append(dolfin_type)
 
-    used_shared_ptr_types.extend(derived for derived, bases in declared_types.items()
+    used_shared_ptr_types.extend(derived for derived, bases in list(declared_types.items())
                                  if any(base in shared_ptr_classes for base in bases))
 
     shared_ptr_declarations = "\n".join([shared_ptr_format % derived \
