@@ -490,7 +490,7 @@ class Expression(object):
                   if (ncells > 0)
                   {
                     CellIterator cell(*mesh);
-                    ndofs_per_cell = dofmap->cell_dimension(cell->index());
+                    ndofs_per_cell = dofmap->num_element_dofs(cell->index());
                   }
                   else
                   {
