@@ -181,8 +181,7 @@ def _check_and_contract_linear_comb(expr, self, multi_index):
     return list(zip(funcs, weights))
 
 class Function(ufl.Coefficient, cpp.Function):
-    """
-    This class represents a function :math:`u_h` in a finite
+    """This class represents a function :math:`u_h` in a finite
     element function space :math:`V_h`, given by
 
     .. math::
@@ -224,6 +223,8 @@ class Function(ufl.Coefficient, cpp.Function):
           <dolfin.functions.functionspace.FunctionSpace>` ``V`` and a
           :py:class:`GenericVector <dolfin.cpp.GenericVector>` ``v``
 
+          *Warning: this constructor is intended for internal libray use only*
+
           .. code-block:: python
 
               g = Function(V, v)
@@ -274,8 +275,8 @@ class Function(ufl.Coefficient, cpp.Function):
                 self.__init_from_function_space(V)
             elif len(args) == 2:
                 # If passing FunctionSpace together with cpp.Function
-                # Attached passed FunctionSpace and initialize the cpp.Function
-                # using the passed Function
+                # Attached passed FunctionSpace and initialize the
+                # cpp.Function using the passed Function
                 other = args[1]
                 if isinstance(other, cpp.Function):
                     self.__init_from_function_space_and_cpp_function(V, other)
@@ -331,11 +332,6 @@ class Function(ufl.Coefficient, cpp.Function):
         cpp.Function.__init__(self, V, other)
         ufl.Coefficient.__init__(self, V.ufl_element(), count=self.id())
 
-    def _sub(self, i, deepcopy = False):
-        cpp.deprecation("Using Function._sub", "1.3.0",
-                        "Use Function.sub instead")
-        self.sub(i, deepcopy)
-
     def sub(self, i, deepcopy = False):
         """
         Return a sub function.
