@@ -112,8 +112,7 @@ def mesh2xml(ifilename, ofilename):
         if not line: break
 
         # Remove newline
-        if line[-1] == "\n":
-            line = line[:-1]
+        line = line.rstrip("\n\r")
 
         # Read dimension
         if  line == "Dimension" or line == " Dimension":
@@ -155,8 +154,7 @@ def mesh2xml(ifilename, ofilename):
             continue
 
         # Remove newline
-        if line[-1] == "\n":
-            line = line[:-1]
+        line = line.rstrip("\n\r")
 
         if state == 0:
             if line == "Dimension" or line == " Dimension":
@@ -269,8 +267,7 @@ def gmsh2xml(ifilename, handler):
     while line:
 
         # Remove newline
-        if line[-1] == "\n":
-            line = line[:-1]
+        line = line.rstrip("\n\r")
 
         # Read dimension
         if line.find("$Elements") == 0:
@@ -366,8 +363,7 @@ def gmsh2xml(ifilename, handler):
             continue
 
         # Remove newline
-        if line[-1] == "\n":
-            line = line[:-1]
+        line = line.rstrip("\n\r")
 
         if state == 0:
             if line == "$MeshFormat":
