@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Jan Blechta, 2012. 
+# Modified by Jan Blechta, 2012.
 #
 # Last changed: 2012-11-22
 
@@ -87,8 +87,8 @@ def write_graph_vertex(ofile, vertex, num_edges, weight = 1):
         (vertex, num_edges, weight))
 
 def write_graph_edge(ofile, v1, v2, weight = 1):
-	 "Write graph edge"
-	 ofile.write("      <edge v1=\"%d\" v2=\"%d\" weight=\"%d\"/>\n" % \
+    "Write graph edge"
+    ofile.write("      <edge v1=\"%d\" v2=\"%d\" weight=\"%d\"/>\n" % \
         (v1, v2, weight))
 
 def write_header_cells(ofile, num_cells):
@@ -103,7 +103,7 @@ def write_footer_cells(ofile):
 
 def write_cell_interval(ofile, cell, n0, n1):
     "Write cell (interval)"
-    ofile.write("      <triangle index=\"%d\" v0=\"%d\" v1=\"%d\"/>\n" % \
+    ofile.write("      <interval index=\"%d\" v0=\"%d\" v1=\"%d\"/>\n" % \
         (cell, n0, n1))
 
 def write_cell_triangle(ofile, cell, n0, n1, n2):
